from typing import Optional, Dict, List


class EventDefinition(object):
    def __init__(self, subject, event_type=None, data=None, headers=None):
        # type: (str, Optional[str], Optional[bytes], List[Dict[str, str]]) -> None
        self.subject = subject
        self.event_type = event_type
        self.data = data
        self.headers = headers
