from redis_lock import Lock


class RedisClusterLock(Lock):
  def __init__(self, redis_client, name, *args, **kwargs):
    super(RedisClusterLock, self).__init__(redis_client, name, *args, **kwargs)
    self._name = f"lock:{{{name}}}"
    self._signal = f"lock-signal:{{{name}}}"
