from messagehandler import MessageHandlerBuilder


class SchedCompMessageHandlerBuilder(MessageHandlerBuilder):
    def build_singletons(self):
        pass

    def build_message_handler(self, message, event):
        pass

    def destroy_message_handler(self, message_handler):
        pass

    def destroy_singletons(self):
        pass
